﻿using System;
using System.Reflection;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;


namespace DerivedEffects
{
  public static class ContentManagerExtensions
  {
    public static T LoadDerivedEffect<T>(this ContentManager content, string assetName)
      where T : Effect
    {
      // get the graphics device
      IGraphicsDeviceService graphicsDeviceService =
          (IGraphicsDeviceService)content.ServiceProvider.GetService(typeof(IGraphicsDeviceService));

      if (graphicsDeviceService == null)
        throw new Exception("Unable to find the graphics device service.");


      // load the compiled effect code
      DerivedEffect effect = content.Load<DerivedEffect>(assetName);

      // find the class constructor
      Type classType = typeof(T);
      ConstructorInfo classConstructor = classType.GetConstructor(
           BindingFlags.Instance | BindingFlags.Public, null,
           new Type[] { graphicsDeviceService.GraphicsDevice.GetType(), effect.CompiledCode.GetType() },
           null);


      if (classConstructor == null)
        throw new Exception(String.Format("Unable to find constructor for {0}.  Your Effect subclass must implement the 2 " +
                                          "parameter constructor with GraphicsDevice and byte[] parameters.", classType.Name));


      // create and return instance
      return (T)classConstructor.Invoke(new object[] { graphicsDeviceService.GraphicsDevice, effect.CompiledCode });
    }
  }
}
