using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Content.Pipeline.Processors;
using Microsoft.Xna.Framework.Content.Pipeline.Graphics;

namespace TangentModelPipeline
{
    /// <summary>
    /// http://msdn.microsoft.com/en-us/library/bb447748.aspx
    /// </summary>
    [ContentProcessor(DisplayName = "Model  Tangent Processor")]
    class ModelProcessorWithTangents : ModelProcessor
    {
        public override ModelContent Process(NodeContent input, ContentProcessorContext context)
        {
            GenerateTangentFramesRecursive(input);
            return base.Process(input, context);
        }

        private void GenerateTangentFramesRecursive(NodeContent node)
        {
            MeshContent mesh = node as MeshContent;
            if (mesh != null)
            {
                MeshHelper.CalculateTangentFrames(mesh, VertexChannelNames.TextureCoordinate(0), VertexChannelNames.Tangent(0), VertexChannelNames.Binormal(0));
            }

            foreach (NodeContent child in node.Children)
            {
                GenerateTangentFramesRecursive(child);
            }
        }
    }
}
