﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using XNATutorial.Entity;

namespace XNATutorial.Camera.Interfaces
{
    interface ICamera : IMoveableEntity
    {
        /// <summary>
        /// Initializes the projection matrix.
        /// </summary>
        /// <param name="fieldOfView">Field of view.</param>
        /// <param name="aspectRatio">Aspect ratio of the window. See: GraphicsDevice.Viewport</param>
        /// <param name="nearClippingPlane">Near clipping plane of the view frustum.</param>
        /// <param name="farClippingPlane">Far clipping plane of the view frustum.</param>
        void Perspective(float fieldOfView, float aspectRatio, float nearClippingPlane, float farClippingPlane);

        /// <summary>
        /// Position of the camera in world-space.
        /// </summary>
        Vector3 Position { get; }

        /// <summary>
        /// Rotation of the camera in object-space.
        /// </summary>
        Quaternion Rotation { get; }

        /// <summary>
        /// The view matrix. Represents the viewing frustum. Needed to determine which objects in a 3D scene are displayed on the screen.
        /// </summary>
        Matrix View { get; }

        /// <summary>
        /// The projection matrix. Needed to project a 3D scene on a 2D display plane.
        /// </summary>
        Matrix Projection { get; }
    }
}
