using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XNATutorial.Effects.Interfaces;

namespace XNATutorial.Effects
{
	class EnvironmentEffect : BaseEffect
	{
		public enum EffectTechniques
		{
			EnvironmentMapTechnique
		}

		#region Properties

		public Vector3 LightDirection { get; set; }
		public Vector3 LightColor { get; set; }
		public Vector3 AmbientColor { get; set; }
		public Texture2D Texture { get; set; }
        public TextureCube EnvironmentMap { get; set; }
		public bool TextureEnabled { get; set; }

		public EffectTechniques Technique { get; set; }

		#endregion

		public EnvironmentEffect(GraphicsDevice graphicsDevice, byte[] effectCode) : base(graphicsDevice, effectCode)
		{
			Name = "Environment";
			LightDirection = new Vector3(0.5773503f, 0.5773503f, 0.5773503f);
			LightColor = new Vector3(0.6f, 0.5f, 0.4f);
			AmbientColor = new Vector3(0.1f, 0.15f, 0.25f);
			TextureEnabled = false;
		}

		protected override void OnApply()
		{
			Parameters["World"].SetValue(World);
			Parameters["View"].SetValue(View);
			Parameters["Projection"].SetValue(Projection);
			Parameters["LightDirection"].SetValue(LightDirection);
			Parameters["LightColor"].SetValue(LightColor);
			Parameters["AmbientColor"].SetValue(AmbientColor);
			Parameters["Texture"].SetValue(Texture);
			Parameters["EnvironmentMap"].SetValue(EnvironmentMap);
			Parameters["TextureEnabled"].SetValue(TextureEnabled);

			switch (Technique)
			{
				case EffectTechniques.EnvironmentMapTechnique:
					CurrentTechnique = Techniques["EnvironmentMapTechnique"];
					break;
				default:
					CurrentTechnique = Techniques["EnvironmentMapTechnique"];
					break;
			}
		}
	}
}
