using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XNATutorial.Effects.Interfaces;

namespace XNATutorial.Effects
{
	class GlossEffect : BaseEffect
	{
		public enum EffectTechniques
		{
			SpecularLight
		}

		#region Properties

		public Vector4 LightDirection { get; set; }
		public Color DiffuseColor { get; set; }
		public Color SpecularLight { get; set; }
		public Color AmbientColor { get; set; }
		public float SpecularPower { get; set; }
		public Texture2D DiffuseTexture { get; set; }
		public Texture2D GlossTexture { get; set; }

		public EffectTechniques Technique { get; set; }

		#endregion

		public GlossEffect(GraphicsDevice graphicsDevice, byte[] effectCode) : base(graphicsDevice, effectCode)
		{
			Name = "Gloss";
			LightDirection = new Vector4(1f, 1f, 1f, 1f);
			DiffuseColor = new Color(1f, 1f, 1f, 1f);
			SpecularLight = new Color(1f, 1f, 1f, 1f);
			AmbientColor = new Color(0.07f, 0.07f, 0.07f, 0.07f);
			SpecularPower = 55f;
		}

		protected override void OnApply()
		{
			Parameters["matWorldViewProj"].SetValue(World * View * Projection);
			Parameters["matWorld"].SetValue(World);
			Parameters["vecEye"].SetValue(new Vector4(View.M41, View.M42, View.M43, 1.0f));
			LightDirection.Normalize();
			Parameters["vecLightDir"].SetValue(LightDirection);
			Parameters["vDiffuseColor"].SetValue(DiffuseColor.ToVector4());
			Parameters["vSpecularColor"].SetValue(SpecularLight.ToVector4());
			Parameters["vAmbient"].SetValue(AmbientColor.ToVector4());
			Parameters["SpecExpon"].SetValue(SpecularPower);
			Parameters["ColorMap"].SetValue(DiffuseTexture);
			Parameters["GlossMap"].SetValue(GlossTexture);

			switch (Technique)
			{
				case EffectTechniques.SpecularLight:
					CurrentTechnique = Techniques["SpecularLight"];
					break;
				default:
					CurrentTechnique = Techniques["SpecularLight"];
					break;
			}
		}
	}
}
