using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XNATutorial.Effects.Interfaces;

namespace XNATutorial.Effects
{
	class GoochEffect : BaseEffect
	{
		public enum EffectTechniques
		{
			Main
		}

		#region Properties

		public Vector3 Lamp0Position { get; set; }
		public Color Lamp0 { get; set; }
		public Color AmbientLight { get; set; }
		public Color GoochWarmTone { get; set; }
		public Color GoochCoolTone { get; set; }
		public float GlossTop { get; set; }
		public float Specular { get; set; }
		public float GlossFalloff { get; set; }
		public float GlossEdge { get; set; }
		public float SpecularPower { get; set; }
		public Color Surface { get; set; }

		public EffectTechniques Technique { get; set; }

		#endregion

		public GoochEffect(GraphicsDevice graphicsDevice, byte[] effectCode) : base(graphicsDevice, effectCode)
		{
			Name = "Gooch";
			Lamp0Position = new Vector3(-0.5f, 2f, 1.25f);
			Lamp0 = new Color(1f, 1f, 1f);
			AmbientLight = new Color(0.07f, 0.07f, 0.07f);
			GoochWarmTone = new Color(0.5f, 0.4f, 0.05f);
			GoochCoolTone = new Color(0.05f, 0.05f, 0.6f);
			GlossTop = 0.65f;
			Specular = 0.4f;
			GlossFalloff = 0.25f;
			GlossEdge = 0.25f;
			SpecularPower = 5f;
			Surface = new Color(1f, 1f, 1f);
		}

		protected override void OnApply()
		{
			Parameters["WorldITXf"].SetValue(Matrix.Transpose(Matrix.Invert(World)));
			Parameters["WvpXf"].SetValue(World * View * Projection);
			Parameters["WorldXf"].SetValue(World);
			Parameters["ViewIXf"].SetValue(Matrix.Invert(View));
			Parameters["Lamp0Pos"].SetValue(Lamp0Position);
			Parameters["Lamp0Color"].SetValue(Lamp0.ToVector3());
			Parameters["AmbiColor"].SetValue(AmbientLight.ToVector3());
			Parameters["WarmColor"].SetValue(GoochWarmTone.ToVector3());
			Parameters["CoolColor"].SetValue(GoochCoolTone.ToVector3());
			Parameters["GlossTop"].SetValue(GlossTop);
			Parameters["Ks"].SetValue(Specular);
			Parameters["GlossDrop"].SetValue(GlossFalloff);
			Parameters["GlossEdge"].SetValue(GlossEdge);
			Parameters["SpecExpon"].SetValue(SpecularPower);
			Parameters["SurfaceColor"].SetValue(Surface.ToVector3());

			switch (Technique)
			{
				case EffectTechniques.Main:
					CurrentTechnique = Techniques["Main"];
					break;
				default:
					CurrentTechnique = Techniques["Main"];
					break;
			}
		}
	}
}
