using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using XNATutorial.Effects.Interfaces;

namespace XNATutorial.Effects
{
	class PhongEffect : BaseEffect
	{
		public enum EffectTechniques
		{
			Main
		}

		#region Properties

		public Vector3 Lamp0Position { get; set; }
		public Color Lamp0 { get; set; }
		public Color AmbientLight { get; set; }
		public float Specular { get; set; }
		public float SpecularPower { get; set; }
		public Texture2D DiffuseTexture { get; set; }

		public EffectTechniques Technique { get; set; }

		#endregion

		public PhongEffect(GraphicsDevice graphicsDevice, byte[] effectCode) : base(graphicsDevice, effectCode)
		{
			Name = "Phong";
			Lamp0Position = new Vector3(-0.5f, 2f, 1.25f);
			Lamp0 = new Color(1f, 1f, 1f);
			AmbientLight = new Color(0.07f, 0.07f, 0.07f);
			Specular = 0.4f;
			SpecularPower = 55f;
		}

		protected override void OnApply()
		{
			Parameters["WorldITXf"].SetValue(Matrix.Transpose(Matrix.Invert(World)));
			Parameters["WvpXf"].SetValue(World * View * Projection);
			Parameters["WorldXf"].SetValue(World);
			Parameters["ViewIXf"].SetValue(Matrix.Invert(View));
			Parameters["Lamp0Pos"].SetValue(Lamp0Position);
			Parameters["Lamp0Color"].SetValue(Lamp0.ToVector3());
			Parameters["AmbiColor"].SetValue(AmbientLight.ToVector3());
			Parameters["Ks"].SetValue(Specular);
			Parameters["SpecExpon"].SetValue(SpecularPower);
			Parameters["ColorTexture"].SetValue(DiffuseTexture);

			switch (Technique)
			{
				case EffectTechniques.Main:
					CurrentTechnique = Techniques["Main"];
					break;
				default:
					CurrentTechnique = Techniques["Main"];
					break;
			}
		}
	}
}
