﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace XNATutorial.Entity
{
    public interface IMoveableEntity
    {
        /// <summary>
        /// Pans the entity along the X, Y, and Z directions.
        /// </summary>
        /// <param name="axis">The direction to pan the entity.</param>
        /// <param name="distance">The amount of X, Y, and Z distance to pan the entity. Units are assumed to be world units.</param>
        void Translate(Vector3 axis, float distance);

        /// <summary>
        /// Rotates the entity along an arbitrary axis, relative to the position of the entity: object-space, not the world-space.
        /// </summary>
        /// <param name="axis">Ccontaining the axis along which to rotate the entity.</param>
        /// <param name="degrees">The number of degrees to rotate the entity by.</param>
        void Rotate(Vector3 axis, float degrees);

        /// <summary>
        /// Rotates the entity along an arbitrary axis, relative to the position of the entity: object-space, not the world-space.
        /// </summary>
        /// <param name="yaw">The number of degrees to rotate the entity arouhnd its y-axis.</param>
        /// <param name="pitch">The number of degrees to rotate the entity arouhnd its x-axis.</param>
        /// <param name="roll">The number of degrees to rotate the entity arouhnd its z-axis..</param>
        void Rotate(float yaw, float pitch, float roll);
    }
}
