﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using XNATutorial.Entity;
using Microsoft.Xna.Framework.Input;

namespace XNATutorial.Input
{
    class TentacleEntityInputComponent : GameComponent
    {
        /// <summary>
        /// Moveable entity object controlled by the user.
        /// </summary>
        public TentacleEntityComponent Entity { get; set; }

        private KeyboardState currentKeyboardState;
        private KeyboardState previousKeyboardState;

        /// <summary>
        /// Creates a new instance of this class.
        /// </summary>
        /// <param name="game">Game that the game component should be attached to.</param>
        /// <param name="entity">Entity controlled by the user.</param>
        public TentacleEntityInputComponent(GameApp game, TentacleEntityComponent entity) : base(game) 
        {
            Entity = entity;

            currentKeyboardState = Keyboard.GetState();
        }

        /// <summary>
        /// Allows the game component to update itself. Catch user input and move the entity.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            previousKeyboardState = currentKeyboardState;
            currentKeyboardState = Keyboard.GetState();

            var translationStep = 0.005f * gameTime.ElapsedGameTime.Milliseconds;

            if (currentKeyboardState.IsKeyDown(Keys.Up))
            {
                Entity.Translate(Vector3.Up, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.Down))
            {
                Entity.Translate(Vector3.Down, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.Left))
            {
                Entity.Translate(Vector3.Left, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.Right))
            {
                Entity.Translate(Vector3.Right, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.PageUp))
            {
                Entity.Translate(Vector3.Forward, translationStep);
            }

            if (currentKeyboardState.IsKeyDown(Keys.PageDown))
            {
                Entity.Translate(Vector3.Backward, translationStep);
            }

            if (currentKeyboardState.IsKeyUp(Keys.Space) && previousKeyboardState.IsKeyDown(Keys.Space))
            {
                Entity.SwitchNextEffect();
            }

            base.Update(gameTime);
        }
    }
}
