float4x4 matWorldViewProj : WorldViewProj;	
float4x4 matWorld : World;	
float4 vecEye : View;

float4 vecLightDir : Normal <
    string UIName =  "Light Direction";
> = {1.0f,1.0f,1.0f,1.0f};
float4 vDiffuseColor : Color <
    string UIName =  "Diffuse Color";
    string UIWidget = "Color";
> = {1.0f,1.0f,1.0f,1.0f};
float4 vSpecularColor : Color <
    string UIName =  "Specular Light";
    string UIWidget = "Color";
> = {1.0f,1.0f,1.0f,1.0f};
float4 vAmbient : Color <
    string UIName =  "Ambient Color";
    string UIWidget = "Color";
> = {0.07f,0.07f,0.07f,0.07f};
float SpecExpon : SpecularPower <
    string UIWidget = "slider";
    float UIMin = 1.0;
    float UIMax = 128.0;
    float UIStep = 1.0;
    string UIName =  "Specular Power";
> = 55.0;

texture ColorMap : DIFFUSE <
    string ResourceName = "default_color.dds";
    string UIName =  "Diffuse Texture";
    string ResourceType = "2D";
>;
sampler ColorMapSampler = sampler_state
{
   Texture = <ColorMap>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture GlossMap <
    string ResourceName = "default_bump_normal.dds";
    string UIName =  "Gloss Texture";
    string ResourceType = "2D";
>;
sampler GlossMapSampler = sampler_state
{
   Texture = <GlossMap>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

struct OUT
{
    float4 Pos  : POSITION;
    float2 Tex : TEXCOORD0;
    float3 L : TEXCOORD1;
    float3 N : TEXCOORD2;
    float3 V : TEXCOORD3;
};


OUT VS(float4 Pos : POSITION, float3 N : NORMAL, float2 Tex: TEXCOORD0)
{
    OUT Out = (OUT)0;      
    
    Out.Pos = mul(Pos, matWorldViewProj);	
    Out.N = mul(N, matWorld);				
    
    float4 PosWorld = mul(Pos, matWorld);	
    
    Out.L = vecLightDir;
    Out.V = vecEye - PosWorld;
    Out.Tex = Tex;
    
   return Out;
}


float4 PS(float2 Tex: TEXCOORD0, float3 L: TEXCOORD1, float3 N : TEXCOORD2, 
			float3 V : TEXCOORD3) : COLOR
{   
    float4 GlossMapColor = tex2D(GlossMapSampler, Tex);
    float4 ColorMapColor = tex2D(ColorMapSampler, Tex);
    float3 Normal = normalize(N);
    float3 LightDir = normalize(L);
    float3 ViewDir = normalize(V);    
    
    float Diff = saturate(dot(Normal, LightDir)); 
    
    // R = 2 * (N.L) * N  L
    float3 Reflect = normalize(2 * Diff * Normal - LightDir);  
    float Specular = pow(saturate(dot(Reflect, ViewDir)), SpecExpon); // R.V^n

    //The "magic" happens here. Use the glosscolor and multiply it with the specular. The gloss color
    // functions like the intensity of the specular highlight.
    Specular = Specular*GlossMapColor.x;

    // I = A + Dcolor * Dintensity * N.L + Scolor * Sintensity * (R.V)n
    return vAmbient + vDiffuseColor * ColorMapColor * Diff + vSpecularColor * Specular; 
}


technique SpecularLight
{
    pass P0
    {
		Sampler[0] = (GlossMapSampler);	
		Sampler[1] = (ColorMapSampler);	
		
        // compile shaders
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}
